@@ -1,8 +1,7 @@
 import types
-from warnings import warn
 import pandas as pd
 from deprecated import deprecated
-from typing import Iterator, List, Union, Tuple, Protocol, runtime_checkable, Optional
+from typing import Iterator, List, Union, Tuple, Protocol, runtime_checkable, Optional, Any
 import pyterrier as pt
 
 LAMBDA = lambda:0  # noqa: E731 LAMBDA is used for the is_lambda method below, so the type is important
@@ -25,16 +24,11 @@ def get_transformer(v, stacklevel=1):
     if is_transformer(v):
         return v
     if is_lambda(v):
-        warn('Coercion of a lambda into a transformer is deprecated; use a pt.apply instead', stacklevel=stacklevel, category=DeprecationWarning)
-        from .apply_base import ApplyGenericTransformer
-        return ApplyGenericTransformer(v)
+        raise ValueError('Coercion of a lambda into a transformer is no longer supported; use a pt.apply instead')
     if is_function(v):
-        from .apply_base import ApplyGenericTransformer
-        warn('Coercion of a function (called "%s") into a transformer is deprecated; use a pt.apply instead' % v.__name__, stacklevel=stacklevel, category=DeprecationWarning)
-        return ApplyGenericTransformer(v)
+        raise ValueError('Coercion of a function (called "%s") into a transformer is no longer supported; use a pt.apply instead' % v.__name__)
     if isinstance(v, pd.DataFrame):
-        warn('Coercion of a dataframe into a transformer is deprecated; use a pt.Transformer.from_df() instead', stacklevel=stacklevel, category=DeprecationWarning)
-        return SourceTransformer(v)
+        raise ValueError('Coercion of a dataframe into a transformer is no longer supported; use a pt.Transformer.from_df() instead')
     raise ValueError("Passed parameter %s of type %s cannot be coerced into a transformer" % (str(v), type(v)))
 
 
@@ -72,14 +66,14 @@ class Transformer:
         return IdentityTransformer()
 
     @staticmethod
-    def from_df(input : pd.DataFrame, uniform=False) -> 'Transformer':
+    def from_df(input : pd.DataFrame, uniform : bool = False) -> 'Transformer':
         """
         Instantiates a transformer from an input dataframe. Some rows from the input dataframe are returned
         in response to a query on the :meth:`transform` method. Depending on the value `uniform`, the dataframe
         passed as an argument to :meth:`transform` can affect this selection.
-        Arguments:
-                input(DataFrame): a dataframe to store and return, based on setting of `uniform`.
-                uniform(bool): If True, input will be returned in its entirety each time, else rows from input that match the qid values from the argument dataframe.
+
+        :param input: a dataframe to store and return, based on setting of `uniform`.
+        :param uniform: If True, input will be returned in its entirety each time, else rows from input that match the qid values from the argument dataframe.
         
         """
         if uniform:
@@ -99,12 +93,8 @@ class Transformer:
                 When :meth:`transform` is not implemented, the default implementation runs :meth:`transform_iter` and
                 converts the output to a ``DataFrame``.
 
-            Arguments:
-                inp(``pd.DataFrame``): The input to the transformer (e.g., queries, documents, results, etc.)
-
-            Returns:
-                The output of the transformer (e.g., result of retrieval, re-writing, re-ranking, etc.)
-
+            :param inp: The input to the transformer (e.g., queries, documents, results, etc.)
+            :return: The output of the transformer (e.g., result of retrieval, re-writing, re-ranking, etc.)
             :rtype: ``pd.DataFrame``
         """
         # We should have no recursive transform <-> transform_iter problem, due to the __new__ check.
@@ -127,12 +117,8 @@ class Transformer:
                 When :meth:`transform_iter` is not implemented, the default implementation runs :meth:`transform` and
                 converts the output to an iterable.
 
-            Arguments:
-                inp(``Iterable[Dict]``): The input to the transformer (e.g., queries, documents, results, etc.)
-
-            Returns:
-                The output of the transformer (e.g., result of retrieval, re-writing, re-ranking, etc.)
-
+            :param inp(``Iterable[Dict]``): The input to the transformer (e.g., queries, documents, results, etc.)
+            :return: The output of the transformer (e.g., result of retrieval, re-writing, re-ranking, etc.)
             :rtype: ``Iterable[Dict]``
         """
         # We should have no recursive transform <-> transform_iter problem, due to the __new__ check.
@@ -147,14 +133,9 @@ class Transformer:
             - Otherwise, invokes :meth:`transform_iter` and returns a generic iterable (returning whatever type is
               returned from :meth:`transform_iter()`.)
 
-            Arguments:
-                inp(``pd.DataFrame``, ``Iterable[Dict]``, ``List[Dict]``): The input to the transformer (e.g., queries,
-                    documents, results, etc.)
-
-            Returns:
-                The output of the transformer (e.g., result of retrieval, re-writing, re-ranking, etc.) as the same
+            :param inp: The input to the transformer (e.g., queries, documents, results, etc.)
+            :return: The output of the transformer (e.g., result of retrieval, re-writing, re-ranking, etc.) as the same
                 type as the input.
-
             :rtype: ``pd.DataFrame``, ``Iterable[Dict]``, ``List[Dict]``
         """
         if isinstance(inp, pd.DataFrame):
@@ -170,9 +151,8 @@ class Transformer:
             The input dataframe is grouped into batches of batch_size queries, and a generator
             returned, such that :meth:`transform` is only executed for a smaller batch at a time. 
 
-            Arguments:
-                input(DataFrame): a dataframe to process
-                batch_size(int): how many input instances to execute in each batch. Defaults to 1.
+            :param input: a dataframe to process
+            :param batch_size: how many input instances to execute in each batch. Defaults to 1.
             
         """
         docno_provided = "docno" in input.columns
@@ -204,14 +184,14 @@ class Transformer:
     def search(self, query : str, qid : str = "1", sort : bool = True) -> pd.DataFrame:
         """
             Method for executing a transformer (pipeline) for a single query. 
-            Returns a dataframe with the results for the specified query. This
-            is a utility method, and most uses are expected to use the :meth:`transform`
-            method passing a dataframe.
 
-            Arguments:
-                query(str): String form of the query to run
-                qid(str): the query id to associate to this request. defaults to 1.
-                sort(bool): ensures the results are sorted by descending rank (defaults to True)
+            :param query: String form of the query to run
+            :param qid: the query id to associate to this request. defaults to 1.
+            :param sort: ensures the results are sorted by descending rank (defaults to True)
+
+            :return: Returns a dataframe with the results for the specified query. This
+                is a utility method, and most uses are expected to use the :meth:`transform`
+                method passing a dataframe.
 
             Example::
 
@@ -236,19 +216,17 @@ class Transformer:
 
         For instance, a pipeline of transformers can be optimised by fusing adjacent transformers.
 
-        Returns:
-            A new transformer that is equivalent to this transformer, but optimised.
+        :return: A new transformer that is equivalent to this transformer, but optimised.
         """
         return self # by default, nothing to compile
 
-    def parallel(self, N : int, backend='joblib') -> 'Transformer':
+    def parallel(self, N : int, backend : str='joblib') -> 'Transformer':
         """
         Returns a parallelised version of this transformer. The underlying transformer must be "picklable". For more information, see
         :ref:`parallel` documentation.
 
-        Args:
-            N(int): how many processes/machines to parallelise this transformer over. 
-            backend(str): which multiprocessing backend to use. Only two backends are supported, 'joblib' and 'ray'. Defaults to 'joblib'.
+        :param N: how many processes/machines to parallelise this transformer over.
+        :param backend: which multiprocessing backend to use. Only two backends are supported, 'joblib' and 'ray'. Defaults to 'joblib'.
         """
         from .parallel import PoolParallelTransformer
         return PoolParallelTransformer(self, N, backend)
@@ -259,8 +237,7 @@ class Transformer:
             Gets the current value of a particular key of the transformer's configuration state.
             By default, this examines the attributes of the transformer object, using ``hasattr()`` and ``setattr()``.
 
-            Arguments:
-                name: name of parameter
+            :param name: name of parameter
         """
         if hasattr(self, name):
             return getattr(self, name)
@@ -273,9 +250,8 @@ class Transformer:
             Adjusts this transformer's configuration state, by setting the value for specific parameter.
             By default, this examines the attributes of the transformer object, using ``hasattr()`` and ``setattr()``.
 
-            Arguments:
-                name: name of parameter
-                value: current value of parameter
+            :param name: name of parameter
+            :param value: current value of parameter
 
         """
         if hasattr(self, name):
@@ -335,11 +311,6 @@ class Transformer:
     def __hash__(self):
         return hash(repr(self))
 
-class TransformerBase(Transformer):
-    @deprecated(version="0.9", reason="Use pt.Transformer instead of TransformerBase")
-    def __init__(self, *args, **kwargs):
-        super(Transformer, self).__init__(*args, **kwargs)
-
 class Indexer(Transformer):
     def __new__(cls, *args, **kwargs):
         instance = super().__new__(cls, *args, **kwargs)
@@ -356,12 +327,14 @@ class Indexer(Transformer):
             instance.transform = types.MethodType(Transformer.transform, instance)
         return instance
 
-    def index(self, iter : pt.model.IterDict, **kwargs):
+    def index(self, iter : pt.model.IterDict, **kwargs) -> Any:
         """
             Takes an iterable of dictionaries ("iterdict"), and consumes them. The index method may return
             an instance of the index or retriever. This method is typically used to implement indexers that
             consume a corpus (or to consume the output of previous pipeline components that have
             transformer the documents being consumed).
+
+            :param iter: An iterable of dictionaries, each representing a document.
         """
         pass
 
@@ -371,11 +344,6 @@ class Indexer(Transformer):
     def transform_iter(self, inp: pt.model.IterDict) -> pt.model.IterDict:
         raise NotImplementedError('You called `transform_iter()` on an indexer. Did you mean to call `index()`?')
 
-class IterDictIndexerBase(Indexer):
-    @deprecated(version="0.9", reason="Use pt.Indexer instead of IterDictIndexerBase")
-    def __init__(self, *args, **kwargs):
-        super(Indexer, self).__init__(*args, **kwargs)
-
 class Estimator(Transformer):
     """
         This is a base class for things that can be fitted.
@@ -384,19 +352,13 @@ class Estimator(Transformer):
         """
             Method for training the transformer.
 
-            Arguments:
-                topics_or_res_tr(DataFrame): training topics (usually with documents)
-                qrels_tr(DataFrame): training qrels
-                topics_or_res_va(DataFrame): validation topics (usually with documents)
-                qrels_va(DataFrame): validation qrels
+            :param topics_or_res_tr: training topics (usually with documents)
+            :param qrels_tr: training qrels
+            :param topics_or_res_va: validation topics (usually with documents)
+            :param qrels_va: validation qrels
         """
         pass
 
-class EstimatorBase(Estimator):
-    @deprecated(version="0.9", reason="Use pt.Estimator instead of EstimatorBase")
-    def __init__(self, *args, **kwargs):
-        super(Estimator, self).__init__(*args, **kwargs)
-
 class IdentityTransformer(Transformer):
     """
         A transformer that returns exactly the same as its input.
@@ -460,8 +422,7 @@ class SupportsFuseRankCutoff(Protocol):
 
         If the fusion is not possible, `None` should be returned.
 
-        Arguments:
-            k(int): The rank cutoff requested
+        :param k: The rank cutoff requested
         """
 
 
@@ -473,9 +434,8 @@ class SupportsFuseFeatureUnion(Protocol):
         This method should return a new transformer that is equivalent to performing self ** other, or `None`
         if the fusion is not possible.
 
-        Arguments:
-                other(Transformer): transformer to the left or right.
-                is_left(bool): is True if self's features are to the left of other's. Otherwise, self's features are to the right.
+        :param other: transformer to the left or right.
+        :param is_left: is True if self's features are to the left of other's. Otherwise, self's features are to the right.
         """
 
 
@@ -494,12 +454,10 @@ class SupportsFuseLeft(Protocol):
         A fused transformer should be more efficient than the unfused version. For instance, a retriever
         followed by a rank cutoff can be fused to perform the rank cutoff during retrieval.
 
-        Arguments:
-            left(Transformer): transformer to the left.
+        :param left: transformer to the left.
 
-        Returns:
-            A new transformer that is the result of merging this transformer with the left transformer,
-            or none if a merge is not possible.
+        :return: A new transformer that is the result of merging this transformer with the left transformer,
+            or None if a merge is not possible.
         """
 
 
@@ -518,12 +476,10 @@ class SupportsFuseRight(Protocol):
         A fused transformer should be more efficient than the unfused version. For instance, a retriever
         followed by a rank cutoff can be fused to perform the rank cutoff during retrieval.
 
-        Arguments:
-            right(Transformer): transformer to the right in a composed pipeline.
+        :param right: transformer to the right in a composed pipeline.
 
-        Returns:
-            A new transformer that is the result of merging this transformer with the right transformer,
-            or none if a merge is not possible.
+        :return: A new transformer that is the result of merging this transformer with the right transformer,
+            or None if a merge is not possible.
         """
 
 
